
/**********************************************************************************
 Persistence of Vision Ray Tracer Scene Description File
 File name   : UG_LawnMaker_mytest_02.pov
 Version     : 3.8
 Description : Example file to show usage of LawnMaker 1.2 by Uwe Glei
               Example of a repetitive use of a grass patch.
               
 Date        : November 2004 / October 2020
 Author      : Thomas de Groot
 E-mail      : 
 
 Copyright (C) 2004/2020. All rights reserved.
**********************************************************************************/

//+w640 +h450 +a0.3 +am2 +bm2 +bs8 +wt6

#version 3.8;

#include "rand.inc"
#include "transforms.inc"

global_settings {
  assumed_gamma 1.0
  max_trace_level 5
  radiosity {
    pretrace_start 0.08           // start pretrace at this size
    pretrace_end   0.04           // end pretrace at this size
    count 50, 1000                // higher -> higher quality (1..1600) [35]
    nearest_count 10, 5           // higher -> higher quality (1..10) [5]
    error_bound 1                 // higher -> smoother, less accurate [1.8]
    recursion_limit 2             // how much interreflections are calculated (1..5+) [2]
    low_error_factor .3           // reduce error_bound during last pretrace step
    gray_threshold 0.0            // increase for weakening colors (0..1) [0]
    minimum_reuse 0.015           // reuse of old radiosity samples [0.015]
    maximum_reuse 0.1							// new parameter 3.7 [0.2]
    brightness 1                  // brightness of radiosity effects (0..1) [1]

    adc_bailout 0.01/2
    normal off                   // take surface normals into account [off]
    media off                    // take media into account [off]
    always_sample off           // turn sampling in final trace off [on]
    //max_sample 1.0              // maximum brightness of samples
  }
}

sphere {0, 1e4 pigment {rgb 0.5} finish {emission 1.0 diffuse 0.0}}
plane {y, -0.5 pigment {rgb 0.5}}

camera { 
  location  <0, 10, -20> 
  direction 1.5*z
  right     x*image_width/image_height
  angle     50
  look_at   <0.0, 0.0, 0.0>
}

light_source {
  <-30, 30, -30>            
  color rgb <0.98, 0.98, 0.97>*3 
}

light_source {
  <30, 30, 30>            
  color rgb <1, 1, 1>*0.1 
  shadowless
}

//==================================================================================
#include "DrawTheAxes.inc"
#include "lawnmaker.inc"
#include "Halm1.inc"

//==================================================================================
// A defined surface on which to plant the grass:

#declare Scale = <15, 3.0, 15>;

#declare Surface =
#difference {
  intersection {
    height_field {
      png "Spherical_01.png" gamma 1.0
      texture {
        pigment {
          //image_map {png "Spherical_01.png" gamma srgb map_type 0 interpolate 2 once}
          image_map {png "WildGrass_2_UR_512.png" gamma srgb map_type 0 interpolate 2 once}
          //image_map {png "WildGrass_2_UR_512_Height.png" gamma srgb map_type 0 interpolate 2 once}
          rotate x*90 // lay X-Y image map down onto X-Z height field plane
        }
      }
	    translate <-0.5, 0.0, -0.5>
	  }
	  box {<-0.499, -1.0, -0.499>, <0.499, 1.0, 0.499> pigment {srgb <0.6, 0.5, 0.2>}}
    scale Scale
  }
  plane {y, -0.5 pigment {rgb 0}}
}

//==================================================================================
//==================================================================================
// How to use Lawnmaker (Only the necessary values are changed)

//---------------------------------------------------------------------------------(1)
// Size and density of the grass patch:

//#declare LawnStart = <-9, -20.5>;
//#declare LawnStop  = < 9,  20.5>;
#declare LawnStart = <-1.0,-1.0>;
#declare LawnStop  = < 1.0, 1.0>;
#declare LawnStepX = <0.1, 0.1>;
#declare LawnStepZ = <0.1, 0.1>;

//---------------------------------------------------------------------------------(2)
// Parameter controlling the individual sizes of the grass blades (including their absence):

#declare Map = pigment {image_map { png "WildGrass_2_UR_512_Height.png" gamma srgb} rotate 90*x translate <-0.5, 0.0, -0.5> scale <Scale.x*0.01, 1.0, Scale.z*0.01> }
#declare LawnHeight = pigment {rgb 1}
/*pigment {
  pigment_pattern {
    boxed //cylindrical
    pigment_map {
      [0.3 rgb 1]
      [0.7 rgb 0]
    }
    //warp {turbulence 0.6}
    scale 10
  }
  pigment_map {
    [0.0 rgb 1]
    [0.5 Map]
    [0.5 rgb 0]
  }
  scale 1
}*/

//---------------------------------------------------------------------------------(3)
// Parameters allowing positioning on a height_field:
// in this example, the patches are positioned later on the height_field, so the
// these parameters are set to their default values.

//#declare LawnPosStretch = Scale.y;
//#declare LawnPosition   = pigment { image_map { png "Spherical_01.png" gamma srgb} rotate 90*x translate <-0.5, 0.0, -0.5> scale Scale }

//---------------------------------------------------------------------------------(4)
// For LawnDirection and LawnInclination, see lawnmaker.inc
// Otherwise, TypeTurn and TypeIncline below, control the blades.

//---------------------------------------------------------------------------------(5)
// New color map used for the straws:

#declare LawnColor[4] =
pigment {
  spotted turbulence 2 scale 0.5
  colour_map {
    [0.0 rgb <0.95, 0.7, 0.25>]
    [1.0 rgb <0.9, 0.8, 0.3>]
  }
}
 
//---------------------------------------------------------------------------------(6)
// Modelling of the grass patch:

// Suggestion: to save space in the list below, use e.g. #declare h = <0.2,0.2,0.5>;

#declare T = 6;
#declare TypeGeometry = array[T] { Blade1,     Blade2,     Blade3,     Blade4,     Blade5,     Halm1   }
#declare TypeWeight   = array[T] { 4,          4,          4,          3,          3,          0.01     }
#declare TypeSize1    = array[T] { <.7,.6,.7>, <.7,.5,.7>, <.6,.4,.6>, <.6,.4,.6>, <.6,.4,.6>, <1,1,1> }
#declare TypeSize2    = array[T] { <1,1.5,1>,  <1,1.4,1>,  <1,1.2,1>,  <1,1.1,1>,  <1,1.1,1>,  <3,3,3> }
#declare TypeColMap   = array[T] { 0,          0,          1,          2,          2,          4       }
#declare TypeTurn     = array[T] { <0, 360>,   <0, 360>,   <0, 360>,   <0, 360>,   <0, 360>,   <0, 360>}
#declare TypeIncline  = array[T] { <0, 50>,    <0, 30>,    <0, 10>,    <0, 5>,     <-2, 2>,    <-5, 5> }

//---------------------------------------------------------------------------------(7)
// invoking LawnMaker and thus actually producing the grass.
// Using LMRandom enables to generate slightly different patches:

LawnMaker()
#declare GrassPatch_01 = object {GrassPatch};

#declare LMRandom = 125;
LawnMaker()
#declare GrassPatch_02 = object {GrassPatch};

#declare LMRandom = 126;
LawnMaker()
#declare GrassPatch_03 = object {GrassPatch};

// -------- Placing the GrassPatches on the height_field -------------
#declare Seed    = seed(74532);
#declare Spacing = 1.9;
#declare Cnt     = 0;

#declare GrassPatchTotal =
//difference {  //using this slows the render speed dramatically!
  union {
    #declare PosX = -(Scale.x/2)+1;

    #while (PosX <= (Scale.x/2))
      #declare PosZ = -(Scale.z/2)+1;

      #while (PosZ <= (Scale.z/2))
        #declare Norm = <0, 0, 0>;
        #declare Start = <PosX, 10, PosZ>;
        #declare Pos = trace (Surface, Start, -y, Norm);
        //#debug concat("  Pos: ",vstr(3, Pos, ", ", 0, 3),"\n")
        #if (vlength(Norm))
          object {
            #local Rand = RRand(0, 90, Seed);
            #if (Rand < 30)
              GrassPatch_01
            #elseif (Rand > 60)
              GrassPatch_03
            #else
              GrassPatch_02
            #end
            rotate int(RRand(0, 3, Seed))*90*y
            Reorient_Trans(y, Norm)
            translate Pos
          }
          #declare Cnt = Cnt+1;
        #end

        #declare PosZ = PosZ+Spacing;
      #end

      #declare PosX = PosX+Spacing;
    #end
    #debug concat("Placed ", str(Cnt, 0, 0), " patches\n")
  }
/*  union {
    plane {y, 0 rotate 90*x translate -Scale.z*z}
    plane {y, 0 rotate 90*z translate  Scale.x*x}
    pigment {srgb <0.6, 0.5, 0.2>}
  }
}*/

/*
#local Min = min_extent(GrassPatch);
#local Max = max_extent(GrassPatch);
#debug concat("\n  min_extent GrassPatch: ",vstr(3, Min, ", ", 0, 3),"\n")
#debug concat("  max_extent GrassPatch: ",vstr(3, Max, ", ", 0, 3),"\n\n")
*/
//---------------------------------------------------------------------------------(8)
// And here are all the scene objects together; GrassPatch is the grass, obviously ;-)

union {
  object {MyAxis(6, 4, 6, 1) scale 0.05 translate 1*y}
  object {GrassPatchTotal}
  object {Surface}
  rotate 20*y
}


